import ppcardhandler.GenericDataContainer

class Status(GenericDataContainer.GenericDataContainer):
    def __init__(self, name, id, data=None):
        """
        Corresponds to a Status column in a Board in Projectplace
        @param name: Name of Status
        @type name: '<name>'
        @param id: Id of Status
        @type id: <id>
        @param data: Data retrieved from Projectplace API representing a Status
        @type data: {'name':'<name>','id':<id>} [primary data, more will be available]
        """
        self.name = name
        self.id = id
        
        if data:
            data2 = data.copy()
            del data2['id']     # We don't want these to overwrite the self.id
            #del data2['name']  # There is no "name" key, only "title".
            GenericDataContainer.GenericDataContainer.__init__(self,initdict=data2)



class Card(GenericDataContainer.GenericDataContainer):
    def __init__(self, name, id, data=None):
        """
        Corresponds to a Card in Projectplace
        @param name: Name of Card
        @type name: '<name>'
        @param id: Id of Card
        @type id: <id>
        @param data: Data retrieved from Projectplace API representing a Card
        @type data: {'name':'<name>','id':<id>} [primary data, more will be available]
        """
        self.name = name
        self.id = id
        
        if data:
            data2 = data.copy()
            del data2['id']     # We don't want these to overwrite the self.id
            #del data2['name']  # There is no "name" key, only "title".
            GenericDataContainer.GenericDataContainer.__init__(self,initdict=data2)

    @property
    def status(self):
        """
        Return status
        """
        return Status(id=self.progress['id'], name=self.progress['name'], data=self.progress)


    def set_status(self, status):
        """
        Set Status of card in Board
        @param status: The (new) status that the card should have
        @type status: <instance of Status>
        """
        from ppcardhandler.core import set_card_status
        set_card_status(self.id, status.id)


        
class Board(GenericDataContainer.GenericDataContainer):
    def __init__(self, projectid, name, id, data=None):
        """
        Corresponds to a Board in Projectplace
        @param projectid: Id of Project that Board belongs to (needed for creation of new cards via API)
        @type projectid: <project id>
        @param name: Name of Board
        @type name: '<name>'
        @param id: Id of Board
        @type id: <board id>
        @param data: Data retrieved from Projectplace API representing a Board
        @type data: {'name':'<name>','id':<id>} [primary data, more will be available]
        """
        
        self.projectid = projectid
        self.name = name
        self.id = id
        
        if data:
            data2 = data.copy()
            del data2['id']     # We don't want these to overwrite the self.id and self.name
            del data2['name']
            GenericDataContainer.GenericDataContainer.__init__(self,initdict=data2)

        self._cards = None
        self._statuses = None
        
    @property
    def cards(self):
        """
        Return Cards associated with Board
        """
        if self._cards == None:
            # Cards info not yet retrieved from API, do it
            from ppcardhandler.core import get_cards
            data = get_cards(self.id)
            self._cards = dict()
            for entry in data:
                self._cards[entry['id']] = Card(name=entry['title'], id=entry['id'], data=entry)
                
        return self._cards

    def card(self, id=0, name='', create=False):
        """
        Return requested Card
        @param id: Card id [optional]
        @type id: <id>
        @param name: Card name [optional]
        @type name: '<name>'
        @param create: If Card should be created (if it doesn't exist)
        @type create. <Boolean>
        @return: A board
        """
        if id:
            # Fetch existing board by ID
            return self.cards[id]
        
        else:
            # Fetch existing board by NAME
            chkname = name.lower()
            for id in self.cards:
                if self.cards[id].name.lower() == chkname:
                    return self.cards[id]
            # Create board
            if create:
                from ppcardhandler.core import create_card
                entry_data = create_card(projectid=self.projectid, boardid=self.id, name=name)
                self._cards[entry_data['id']] = Card(name=entry_data['title'], id=entry_data['id'], data=entry_data)
                return self._cards[entry_data['id']]
            
            # Nothing to return
            else:
                return None

    
    @property 
    def statuses(self):
        """
        Return all available Statuses for this board
        @return: List of Statuses
        """
        if self._statuses == None:
            # Organise status data in more structured format
            self._statuses = list()
            for entry in self.progresses:
                self._statuses.append(Status(name=entry['name'], id=entry['id'], data=entry))
        return self._statuses
    

    def status(self, name, exactmatchonly=True):
        """
        Return status
        @param name: Name of Status
        @type name: '<name>'
        @return: Status object with a corresponding name
        @param exactmatchonly: Option to require less than exact match
        @type exactmatchonly: <Boolean>
        @return: Status object or None
        """
          
        for status in self.statuses:
            if status.identifier_matches(name, exactmatchonly=exactmatchonly):
                return status
        return None



class Project(GenericDataContainer.GenericDataContainer):
    def __init__(self, name, id, data=None):
        """
        Corresponds to a project in Projectplace
        @param name: Name of Project
        @type name: '<name>'
        @param id: Id of Project
        @type id: <id>
        @param data: Data representing a project. None to fetch from API, {} to not fetch.
        @type data: Example {'name':'<name>','id':<id>} [primary data, more will be available]
        """

        if data==None:
            # We will only fetch if data is None. In case of an empty dict
            # we do not fetch (assumes there is no relevant data to fetch). 
            # This can be utilized to speed up init process. If you have no
            # need for this metadata (name, description, etc.) but are
            # interested in Boards & Cards methods then use data={}
            
            # Fetch meta data for Project from API.
            from ppapiaccess import get_projects
            
            data = get_projects(name=name, id=id)
            if not data:
                errormsg = """No Project could be fetched from Projectplace.com. \
Neither supplied name ("%s") or id ("%s") correspond to any \
accessible Project. Please verify these values and try again."""%(name, id)
                raise ValueError(errormsg)
            
            if not id:
                id = data['id']
            if not name:
                name = data['name']
            
        self.name = name
        self.id = id

        if data:
            # Populate current object with attributes corresponding to content in data
            data2 = data.copy()
            
            # We don't want to overwrite the self.id and self.name
            del data2['id']     
            del data2['name']

            # Call parent                        
            GenericDataContainer.GenericDataContainer.__init__(self,initdict=data2)
        
        self._boards = None
   

    @property
    def boards(self):
        """
        Return Boards associated with project
        """
        if self._boards == None:
            # Boards info not yet retrieved from API, do it
            from ppcardhandler.core import get_boards
            data = get_boards(self.id)
            self._boards = dict()
            for entry in data:
                new_board = Board(projectid=self.id, name=entry['name'], id=entry['id'], data=entry)
                self._boards[new_board.id] = new_board
                
        return self._boards

                
    def board(self, id=0, name='', create=False):
        """
        Return requested board
        @param id: Board id [optional]
        @type id: <id>
        @param name: Board name [optional]
        @type name: '<name>'
        @param create: If board should be created (if it doesn't exist)
        @type create. <Boolean>
        @return: A board
        """
        if id:
            # Fetch existing board by ID
            return self.boards[id]
        
        else:
            # Fetch existing board by NAME
            chkname = name.lower()
            for id in self.boards:
                if self.boards[id].name.lower() == chkname:
                    return self.boards[id]
            # Create board
            if create:
                from ppcardhandler.core import create_board
                entry_data = create_board(self.id, name)
                self._boards[entry_data['id']] = Board(projectid=self.id, name=entry_data['name'], id=entry_data['id'], data=entry_data)
                return self._boards[entry_data['id']]
            
            # Nothing to return
            else:
                return None



